package com.gyee.alarm.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gyee.alarm.init.CacheContext;
import com.gyee.alarm.model.auto.ProBasicEquipment;
import com.gyee.alarm.model.auto.ProBasicFeatureStat;
import com.gyee.alarm.model.auto.ProEconAlarmType;
import cn.hutool.core.bean.BeanUtil;
import com.gyee.alarm.model.auto.ProBasicFeatureStat;
import com.gyee.alarm.model.vo.AlarmType;
import com.gyee.alarm.service.auto.IAlarmTsService;
import com.gyee.alarm.service.auto.IProBasicFeatureStatService;
import com.gyee.alarm.util.DateUtils;
import com.gyee.alarm.util.SnowflakeGenerator;
import com.gyee.alarm.util.StringUtils;


import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.sql.Timestamp;
import java.util.*;
import java.util.stream.Collectors;

@Service

public class FeatureStatService {
    @Resource
    private IAlarmTsService alarmTsService;

    @Resource
    private IProBasicFeatureStatService proBasicFeatureStatService;

    public List<ProBasicFeatureStat> findFeatureStatList(Date recordDate) {


        Calendar c = Calendar.getInstance();
        c.setTime(recordDate);
        Date begin = DateUtils.truncate(c.getTime());
        Date end = DateUtils.addDays(begin, 1);

        Date statDate=DateUtils.truncate(recordDate);

        QueryWrapper<ProBasicFeatureStat> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("stat_date", statDate);
        //判断是否有重复记录，先删除重复记录
        List<String> idls = proBasicFeatureStatService.list(queryWrapper).stream()
                .map(ProBasicFeatureStat::getId)
                .collect(Collectors.toList());

        if (idls.size() > 0) {

            proBasicFeatureStatService.removeByIds(idls);
        }

        List<ProBasicFeatureStat> fsls = alarmTsService.findFeatureStatList(begin.getTime(), end.getTime());

        Map<String, ProBasicFeatureStat> map = new HashMap<>();


        if (!fsls.isEmpty()) {
            for (ProBasicFeatureStat fs : fsls) {
                if (StringUtils.notEmp(fs.getTypeCode()) && CacheContext.alarmTypeMap.containsKey(fs.getTypeCode().toLowerCase())) {
                    ProEconAlarmType at = CacheContext.alarmTypeMap.get(fs.getTypeCode().toLowerCase());
                    fs.setTypeName(at.getName());
                }
                fs.setStatDate(new Timestamp(statDate.getTime()));
                fs.setId(String.valueOf(SnowflakeGenerator.generateId()));
                StringBuilder sb = new StringBuilder();
                sb.append(fs.getWtId()).append("_").append(fs.getTypeCode().toLowerCase());
                map.put(String.valueOf(sb), fs);
            }
        }
        List<ProBasicFeatureStat> vos = new ArrayList<>();
        for (ProBasicEquipment wt : CacheContext.wtls) {
            for (ProEconAlarmType at : CacheContext.alarmTypeList) {

                if(wt.getWindpowerstationId().contains("_FDC_"))
                {
                    if (at.getCategory().equals(AlarmType.FJBJ.getCode())) {
                        setValues(statDate,map, vos, wt, at);
                    }
                }else   if(wt.getWindpowerstationId().contains("_GDC_"))
                {
                    if (at.getCategory().equals(AlarmType.GFBJ.getCode())) {
                        setValues(statDate,map, vos, wt, at);
                    }
                }

            }
        }

        List<ProBasicFeatureStat> templs = new ArrayList<>();
        for (ProBasicFeatureStat vo : vos) {
            templs.add(vo);
            if (templs.size() == 100) {
                proBasicFeatureStatService.saveBatch(templs);
                templs = new ArrayList<>();
            }

        }

        if (!templs.isEmpty()) {
            proBasicFeatureStatService.saveBatch(templs);
        }

        return fsls;
    }

    private static void setValues(Date statDate,Map<String, ProBasicFeatureStat> map, List<ProBasicFeatureStat> vos, ProBasicEquipment wt, ProEconAlarmType at) {

        ProBasicFeatureStat fs = new ProBasicFeatureStat();

        StringBuilder sb = new StringBuilder();
        sb.append(fs.getWtId()).append("_").append(at.getNemCode().toLowerCase());
        if (map.containsKey(String.valueOf(sb))) {
            ProBasicFeatureStat old = map.get(String.valueOf(sb));
            BeanUtil.copyProperties(old, fs);
        } else {
            fs.setStatDate(new Timestamp(statDate.getTime()));
            fs.setId(String.valueOf(SnowflakeGenerator.generateId()));
            fs.setWtId(wt.getId());
            fs.setStationId(wt.getWindpowerstationId());
            fs.setTypeCode(at.getNemCode());
            fs.setTypeName(at.getName());
            fs.setTotal(0l);
            fs.setAvgSeconds(0.0);
            fs.setMaxSeconds(0.0);
            fs.setMinSeconds(0.0);
            fs.setTotalSeconds(0.0);

        }
        vos.add(fs);
    }


}
