package com.gyee.alarm.controller;

import com.gyee.alarm.model.vo.AjaxResult;
import com.gyee.alarm.model.vo.AjaxStatus;
import com.gyee.alarm.model.vo.AlarmVo;
import com.gyee.alarm.service.AlarmHistoryService;

import com.gyee.alarm.util.DateUtils;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;


@Slf4j
@RestController
@RequestMapping("/alarm")
@CrossOrigin
public class AlarmHistoryController {


    Logger logger = LogManager.getLogger(AlarmHistoryController.class);

    @Resource
    private AlarmHistoryService alarmHistoryService;

    @GetMapping(value = "/history/findAlarmlist")
    @ApiOperation(value = "报警信息查询", notes = "报警信息查询")
    @ApiImplicitParams({

            @ApiImplicitParam(name = "begin", value = "开始时间", required = true, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "end", value = "结束时间", required = true, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "alarmType", value = "报警类型 booststation:升压站报警,windturbine:设备报警,custom:自定义报警", required = true, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "stationid", value = "场站编号", required = false, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "deviceid", value = "设备编号", required = false, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "description", value = "描述", required = false, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "modelId", value = "型号", required = false, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "deviceType", value = "报警类型 booststation:升压站报警,windturbine:设备报警", required = false, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "pageNum", value = "页号", required = true, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页显示行数", required = true, dataType = "string", paramType = "query")})
    public AjaxResult findAlarmlist(
            @RequestParam(value = "begin", required = true) String begin,
            @RequestParam(value = "end", required = true) String end,
            @RequestParam(value = "alarmType", required = true) String alarmType,
            @RequestParam(value = "stationid", required = false) String stationid,
            @RequestParam(value = "deviceid", required = false) String deviceid,
            @RequestParam(value = "description", required = false) String description,
            @RequestParam(value = "modelId", required = false) String modelId,
            @RequestParam(value = "deviceType", required = false) String deviceType,
            @RequestParam(value = "pageNum", required = true) Integer pageNum,
            @RequestParam(value = "pageSize", required = true) Integer pageSize
    ) throws Exception {

        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        Date beginDate = df.parse(begin);
        Date endDate = df.parse(end);
        Map<String,Object> map = alarmHistoryService.findAlarmlist(modelId,beginDate.getTime(), endDate.getTime(), alarmType, stationid,deviceid, description,deviceType, pageNum, pageSize);
        return AjaxResult.successData(AjaxStatus.success.code, map);

    }

    @GetMapping(value = "/history/findAlarmByTypelist")
    @ApiOperation(value = "获取升压站、设备、自定义报警报警信息查询", notes = "报警信息查询")
   public AjaxResult findAlarmByTypelist() throws Exception {


        Map<String,Object> map = alarmHistoryService.findAlarmByTypelist();
        return AjaxResult.successData(AjaxStatus.success.code, map);

    }

//
//    @GetMapping(value = "/count/query/new")
//    public List<Map.Entry<String, List<AlarmStat1>>> querynew(
//            @RequestParam(value = "stationid", required = true) String stId,
//            @RequestParam(value = "startdate", required = true) String startDate,
//            @RequestParam(value = "enddate", required = true) String endDate
//    ) throws ParseException {
//
//        List<AlarmStat1> result = new ArrayList<>();
//        result = alarmCountService.statAlarmCount1(stId, startDate, endDate);
//        //添加拦截,查询液压泵运行统计,并将结果合并为 List<AlarmStat1>
//        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
//        Date dtStart = sdf.parse(startDate);
//        Date dtEnd = sdf.parse(endDate);
//        List<AlarmStat1> pressureList = featureStatService.findByCondition(stId, dtStart, dtEnd);
//
//        result.addAll(pressureList);
//
//        for (int i = 0; i < result.size(); i++) {
//            AlarmStat1 data = result.get(i);
//            //根据预警内容获取所属部件code
//            if (alarmInfoCache.getAlertTextToPartMap().containsKey(data.getAlertText())) {
//                String relatedParts = alarmInfoCache.getAlertTextToPartMap().get(data.getAlertText());
//                data.setRelateParts(relatedParts);
//                //根据预警所属部件code获取所属部件名称
//                if (alarmInfoCache.getRelatePartsMap().containsKey(relatedParts)) {
//                    String releatedPartsText = alarmInfoCache.getRelatePartsMap().get(relatedParts);
//                    data.setRelatePartsText(releatedPartsText);
//                }
//            }
//        }
//
//
//  /*      String[] arr = new String[]{
//                "大风引起震动", "机舱震动幅度超限", "断轴或联轴器打滑", "风向突变", "风速突变", "单机风向标故障", "单机风速仪故障", "机组逆功率", "发电机轴A损坏或碳刷室温度过高", "主轴温度温差大于8度", "主轴温度高", "主轴温升大于40度", "齿轮箱油温温度传感器异常值", "齿轮箱温控阀故障或换热器效率低", "齿轮箱轴承温升超过40度", "齿轮箱轴承温升速度超过1.5", "齿轮箱轴承温差大于8度", "发电机轴承温度高", "发电机轴承温升速度大于1.5", "齿轮箱轴承温度高", "机端低电压", "温度测点开路", "停机收浆超时", "液压泵运行", "滤网压差", "偏航", "解缆动作", "变桨通信不良", "叶轮转速超速", "发电机转速超速", "三相电流不平衡"
//        };*/
//
//        //String str = "大风引起震动,机舱震动幅度超限,断轴或联轴器打滑,风向突变,风速突变,单机风向标故障,单机风速仪故障,机组逆功率,发电机轴A损坏或碳刷室温度过高,主轴温度温差大于8度,主轴温度高,主轴温升大于40度,齿轮箱油温温度传感器异常值,齿轮箱温控阀故障或换热器效率低,齿轮箱轴承温升超过40度,齿轮箱轴承温升速度超过1.5,齿轮箱轴承温差大于8度,发电机轴承温度高,发电机轴承温升速度大于1.5,齿轮箱轴承温度高,机端低电压,温度测点开路,停机收浆超时,液压泵运行,滤网压差,偏航,解缆动作,变桨通信不良,叶轮转速超速,发电机转速超速,三相电流不平衡";
//        //  String str  = "发电机轴承温升速度大于1.5,断轴或联轴器打滑,齿轮箱轴承温升超过40度,三相电流不平衡,发电机轴A损坏或碳刷室温度过高,风向突变,发电机转速超速,单机风向标故障,变桨通信不良,机舱震动幅度超限,齿轮箱轴承温升速度超过1.5,发电机轴承温升大于40度,齿轮箱温控阀故障或换热器效率低,滤网压差,解缆动作,液压泵运行,偏航,主轴温度温差大于8度,风速突变,风机偏航过程震动,齿轮箱轴承温差大于8度,叶轮转速超速,单机发电机W2绕组超限报警,单机发电机W2绕组温度传感器异常值,单机发电机W2绕组温度预警,单机发电机U2绕组温度预警,单机发电机U2绕组超限报警,停机收浆超时,单机发电机V2绕组超限报警,单机发电机V2绕组温度预警,单机发电机V2绕组温度传感器异常值,机组逆功率,风机启动过程震动,桨叶角过小,单机风速仪故障,AGC 负荷调节延迟预警,主轴叶轮侧温度温度传感器异常值,主轴温升大于40度,主轴温升速度大于1.5,主轴温度高,主轴齿轮箱侧温度传感器异常值,单机发电机U1绕组温度传感器异常值,单机发电机U1绕组温度预警,单机发电机U1绕组超限报警,单机发电机U2绕组温度传感器异常值,单机发电机V1绕组温度传感器异常值,单机发电机V1绕组温度预警,单机发电机V1绕组超限报警,单机发电机W1绕组温度传感器异常值,单机发电机W1绕组温度预警,单机发电机W1绕组超限报警,单机发电机轴a温度传感器异常值,单机发电机轴a温度超限报警,单机发电机轴a温度预警,单机发电机轴b温度传感器异常值,单机发电机轴b温度超限报警,单机发电机轴b温度预警,单机变桨电池柜温度1温度传感器异常值,单机变桨电池柜温度2温度传感器异常值,单机变桨电池柜温度2超限报警,单机变桨电池柜温度2预警,单机变桨电池柜温度3温度传感器异常值,单机变桨电池柜温度3超限报警,单机变桨电池柜温度3预警,单机齿轮箱油温超限报警,发电机轴a每分钟温升超过1.5,发电机轴承温度高,变桨电容电压低,变桨电容电压高,塔底柜温度传感器异常值,夏季单机齿轮箱油温温度传感器异常值,大风引起震动,暴风天气,机端低电压,机端过电压,机组过功率,机舱温度温度传感器异常值,机舱温度高,桨叶角度差,温度测点开路,石板泉发电机轴承温度超过70度,风机停机过程震动,风机轴a轴b温差大,风机轴a轴b温差大_UP82,齿轮箱油温度高,齿轮箱油温温度传感器异常值,齿轮箱轴承温度高,齿轮箱输入轴1温度传感器异常值,齿轮箱输出轴2温度传感器异常值,齿轮箱高速轴轴温大于90度";
//        String str = "发电机轴承温升速度大于1.5,齿轮箱轴承温升超过40度,三相电流不平衡,发电机轴A损坏或碳刷室温度过高,风向突变,发电机转速超速,单机风向标故障,变桨通信不良,机舱震动幅度超限,齿轮箱轴承温升速度超过1.5,发电机轴承温升大于40度,齿轮箱温控阀故障或换热器效率低,滤网压差,解缆动作,液压泵运行,偏航,主轴温度温差大于8度,风速突变,风机偏航过程震动,齿轮箱轴承温差大于8度,叶轮转速超速,单机发电机W2绕组超限报警,单机发电机W2绕组温度传感器异常值,单机发电机W2绕组温度预警,单机发电机U2绕组温度预警,单机发电机U2绕组超限报警,停机收浆超时,单机发电机V2绕组超限报警,单机发电机V2绕组温度预警,单机发电机V2绕组温度传感器异常值,机组逆功率,风机启动过程震动,桨叶角过小,单机风速仪故障,AGC 负荷调节延迟预警,主轴叶轮侧温度温度传感器异常值,主轴温升大于40度,主轴温升速度大于1.5,主轴温度高,主轴齿轮箱侧温度传感器异常值,单机发电机U1绕组温度传感器异常值,单机发电机U1绕组温度预警,单机发电机U1绕组超限报警,单机发电机U2绕组温度传感器异常值,单机发电机V1绕组温度传感器异常值,单机发电机V1绕组温度预警,单机发电机V1绕组超限报警,单机发电机W1绕组温度传感器异常值,单机发电机W1绕组温度预警,单机发电机W1绕组超限报警,单机发电机轴a温度传感器异常值,单机发电机轴a温度超限报警,单机发电机轴a温度预警,单机发电机轴b温度传感器异常值,单机发电机轴b温度超限报警,单机发电机轴b温度预警,单机变桨电池柜温度1温度传感器异常值,单机变桨电池柜温度2温度传感器异常值,单机变桨电池柜温度2超限报警,单机变桨电池柜温度2预警,单机变桨电池柜温度3温度传感器异常值,单机变桨电池柜温度3超限报警,单机变桨电池柜温度3预警,单机齿轮箱油温超限报警,发电机轴a每分钟温升超过1.5,发电机轴承温度高,变桨电容电压低,变桨电容电压高,塔底柜温度传感器异常值,夏季单机齿轮箱油温温度传感器异常值,大风引起震动,暴风天气,机端低电压,机端过电压,机组过功率,机舱温度温度传感器异常值,机舱温度高,桨叶角度差,温度测点开路,石板泉发电机轴承温度超过70度,风机停机过程震动,风机轴a轴b温差大,风机轴a轴b温差大_UP82,齿轮箱油温度高,齿轮箱油温温度传感器异常值,齿轮箱轴承温度高,齿轮箱输入轴1温度传感器异常值,齿轮箱输出轴2温度传感器异常值,齿轮箱高速轴轴温大于90度";
//
//        List<String> mustCloumList = Arrays.asList(str.split(","));
//        //去重获取所有列
//        List<String> cloumList = result.stream().map(AlarmStat1::getAlertText).collect(Collectors.toList());
//        cloumList = cloumList.stream().distinct().collect(Collectors.toList());
//        //去重复并集
//        cloumList.addAll(mustCloumList);
//        List<String> allCloumList = cloumList.stream().distinct().collect(Collectors.toList());
//
//
//        //根据风机编号分组
//        Map<String, List<AlarmStat1>> alarmStatMapGroupbyWindturbineId = result.stream().collect(Collectors.groupingBy(AlarmStat1::getWindturbineId));
//
//        Map<String, List<AlarmStat1>> resultMap = new TreeMap<>();
//
//        for (Map.Entry<String, List<AlarmStat1>> entry : alarmStatMapGroupbyWindturbineId.entrySet()) {
//            String windturbineId = entry.getKey();
//            String stationId = entry.getValue().get(0).getStationId();
//
//            List<String> lst = entry.getValue().stream().map(AlarmStat1::getAlertText).collect(Collectors.toList());
//            //与总列的差集
//            List<String> subList = allCloumList.stream().filter(item -> !lst.contains(item)).collect(Collectors.toList());
//            //将差集(缺失数据添加到原数据中)
//
//            List<AlarmStat1> dataList = entry.getValue();
//
//            for (String txt : subList) {
//
//                AlarmStat1 stat = new AlarmStat1();
//                stat.setAlertText(txt);
//                stat.setCount(0);
//                stat.setTime(0);
//                stat.setWindturbineId(windturbineId);
//                stat.setStationId(stationId);
//
//                if (alarmInfoCache.getAlertTextToPartMap().containsKey(txt)) {
//                    String relatedParts = alarmInfoCache.getAlertTextToPartMap().get(txt);
//                    stat.setRelateParts(relatedParts);
//                    //根据预警所属部件code获取所属部件名称
//                    if (alarmInfoCache.getRelatePartsMap().containsKey(relatedParts)) {
//                        String releatedPartsText = alarmInfoCache.getRelatePartsMap().get(relatedParts);
//                        stat.setRelatePartsText(releatedPartsText);
//                    }
//                }
//
//                dataList.add(stat);
//            }
//            if (!resultMap.containsKey(windturbineId)) {
//                resultMap.put(windturbineId, dataList);
//            }
//        }
//
//        //根据风机编号进行排序
//        List<Map.Entry<String, List<AlarmStat1>>> list = new ArrayList<Map.Entry<String, List<AlarmStat1>>>(resultMap.entrySet());
//        Collections.sort(list, new Comparator<Map.Entry<String, List<AlarmStat1>>>() {
//            @Override
//            public int compare(Map.Entry<String, List<AlarmStat1>> o1, Map.Entry<String, List<AlarmStat1>> o2) {
//                int num1 = Integer.valueOf(o1.getKey().substring(5));
//                int num2 = Integer.valueOf(o2.getKey().substring(5));
//                int diff = num1 - num2;
//                if (diff > 0) {
//                    return 1;
//                } else if (diff < 0) {
//                    return -1;
//                }
//                return 0; //相等为0
//            }
//        });
//
//        return list;
//
//
//    }

}




